/*  pnp.h : June 5, 1996  */

/* Copyright (C) 1996-1999  Sekhar Bala,
 *                          Ramchander Balasubramanian, and
 *                          Alphax Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __PNP_HPP
#define __PNP_HPP

   #ifndef __GO32__
      #include <asm/io.h>
      #include <asm/bitops.h>
   #else
      #include <inlines/pc.h>
//      #include <bitops.h>
      #define inb    inportb
      #define outb   outportb
      #define ioperm(x,y,z)      1
      #define iopl(x)            1
   #endif

/*-------------------------------------------------------------------------*/


/* general access ports */
#define ADDRESS_ADDR       0x0279
#define WRITEDATA_ADDR     0x0A79

/* min/max read-data ports (bottom 2 bits must be set) */
#define PNP_DATAPORT_MIN   0x0203
#define PNP_DATAPORT_MAX   0x03FF

/* read-data port increments - must be multiple of 4 */
#define READ_ADDR_STEP           8

/* general registers */
#ifndef __GO32__
#define ADDRESS(x)               outb(x, ADDRESS_ADDR)
#define WRITE_DATA(x)            outb(x, WRITEDATA_ADDR)
#define READ_DATA                inb(PNP_port)
#else
#define ADDRESS(x)               outb(ADDRESS_ADDR, x)
#define WRITE_DATA(x)            outb(WRITEDATA_ADDR, x)
#define READ_DATA                inb(PNP_port)
#endif

/* Registers */
#define R_SETRDPORT(x)          (ADDRESS(0x00), WRITE_DATA(((x) >> 2)))
#define R_SERIALISOLATION        ADDRESS(0x01)
#define R_CONFIGCONTROL          ADDRESS(0x02)
#define R_WAKE(x)               (ADDRESS(0x03),WRITE_DATA(x))
#define R_RESOURCEDATA           ADDRESS(0x04)
#define R_STATUS                 ADDRESS(0x05)
#define R_CARDSELECTNUMBER       ADDRESS(0x06)
#define R_LOGICALDEVICENUMBER    ADDRESS(0x07)
#define R_ACTIVATE               ADDRESS(0x30)
#define R_IORANGECHECK           ADDRESS(0x31)

/* Bits */
#define CONFIG_RESET             0x01
#define CONFIG_WAIT4KEY          0x02
#define CONFIG_RESET_CSN         0x04
#define CONFIG_RESET_DRV         0x07

/* General Tags */
#define R_end_TAG                0x79

/* Short  Tags */
#define _short_TAG(v,t)          ( ((v) & shortmask_TAG) == (t) )
#define shortmask_TAG            (0x0F << 3)

#define sunknown_TAG             (0x00 << 3)
#define pnpverno_TAG             (0x01 << 3)
#define logdevid_TAG             (0x02 << 3)
#define cmptdevid_TAG            (0x03 << 3)
#define irq_TAG                  (0x04 << 3)
#define dma_TAG                  (0x05 << 3)
#define startdep_TAG             (0x06 << 3)
#define enddep_TAG               (0x07 << 3)
#define ioport_TAG               (0x08 << 3)
#define fixedio_TAG              (0x09 << 3)
#define rsvdA_TAG                (0x0A << 3)
#define rsvdB_TAG                (0x0B << 3)
#define rsvdC_TAG                (0x0C << 3)
#define rsvdD_TAG                (0x0D << 3)
#define svendor_TAG              (0x0E << 3)
#define end_TAG                  (0x0F << 3)

/* Long  Tags */
#define longmask_TAG             0x80
#define lunknown_TAG             0x00
#define mem_TAG                  0x01
#define ansistr_TAG              0x02
#define unicodestr_TAG           0x03
#define lvendor_TAG              0x04
#define mem32_TAG                0x05
#define fixedmem32_TAG           0x06

#define SERIAL_LEN                9
#define MAX_CARDS                16


/*-------------------------------------------------------------------------*/

#define DL_MEM_INC         8
#define DL_MEM_BASE        0x40
#define DL_MEM_LIMIT       0x5F
#define DL_MEM_LEN         (((DL_MEM_LIMIT - DL_MEM_BASE)+1)/DL_MEM_INC)

typedef struct {
   INT_U2      base;       // base >> 8
   UCHAR       flag;
   INT_U2      limit;      // range or upper >> 8
} MEM_TYP;

// carefull these are faulty and codified execptions
#define DL_MEM32_INC       16
#define DL_MEM32_BASE      0x70
#define DL_MEM32_LIMIT     0xAF
#define DL_MEM32_LEN       (((DL_MEM32_LIMIT - DL_MEM32_BASE)+1)/DL_MEM32_INC)

typedef struct {
   INT_U4      base;       // base
   UCHAR       flag;
   INT_U4      limit;      // range or upper
} MEM32_TYP;

#define DL_IO_INC         2
#define DL_IO_BASE        0x60
#define DL_IO_LIMIT       0x6F
#define DL_IO_LEN         (((DL_IO_LIMIT - DL_IO_BASE)+1)/DL_IO_INC)

#define DL_IRQ_INC         2
#define DL_IRQ_BASE        0x70
#define DL_IRQ_LIMIT       0x73
#define DL_IRQ_LEN         (((DL_IRQ_LIMIT - DL_IRQ_BASE)+1)/DL_IRQ_INC)

#define DL_DMA_INC         1
#define DL_DMA_BASE        0x74
#define DL_DMA_LIMIT       0x75
#define DL_DMA_LEN         (((DL_DMA_LIMIT - DL_DMA_BASE)+1)/DL_DMA_INC)

#define DL_VENDOR_INC      1
#define DL_VENDOR_BASE     0xF0
#define DL_VENDOR_LIMIT    0xFE
#define DL_VENDOR_LEN      ((DL_VENDOR_LIMIT - DL_VENDOR_BASE)+1)

typedef struct {
   INT_2       dsn;                             // device select number
   UCHAR       active;                          // is device active ?
   UCHAR       rangecheck;                      // current range-check value
   CHAR        id[8];                           // serial key
   MEM_TYP     mem[DL_MEM_LEN];                 // 8/16 bit mem
   MEM32_TYP   mem32[DL_MEM32_LEN];             // 32 bit mem
   INT_U2      ioport[DL_IO_LEN];               // I/O base
   UCHAR       irq[DL_IRQ_LEN][2];              // IRQ / FLG
   UCHAR       dma[DL_DMA_LEN];                 // (4=disable)
   UCHAR       vendor[DL_VENDOR_LEN];           // vendor private data
} PNPDEV_TYP;

#define CL_VENDOR_BASE     0x20
#define CL_VENDOR_LIMIT    0x2F
#define CL_VENDOR_LEN      ((CL_VENDOR_LIMIT - CL_VENDOR_BASE)+1)

typedef struct {
   INT_U2         csn;
   UCHAR          serial[SERIAL_LEN];
   INT_2          data_len;
   UCHAR         *data;
   CHAR           id[8];
   INT_U2         unique;
   UCHAR          vendor[CL_VENDOR_LEN];
   INT_2          dev_len;
   PNPDEV_TYP   (*dev)[];
} PNPCARD_TYP;

EXTERN BOOLEAN             verbose;
EXTERN INT_2               PNP_port;
EXTERN INT_2               PNP_csn;
EXTERN INT_2               PNP_csn_idx;
EXTERN INT_2               PNP_device;

EXTERN INT_2               PNP_last_csn;
EXTERN INT_2               PNP_cards_len;
EXTERN PNPCARD_TYP         PNP_cards[MAX_CARDS+1];

/*-------------------------------------------------------------------------*/


/* pnp01.cpp */
EXTERN BOOLEAN    init_PNP         ( VOID );
EXTERN BOOLEAN    defaultport_PNP  ( INT_2 port=PNP_DATAPORT_MIN );
EXTERN VOID       statuswait_PNP   ( VOID );
EXTERN BOOLEAN    current_PNP      ( INT_2 ucsn=-1 );
EXTERN BOOLEAN    testcsn_PNP      ( BOOLEAN seterr=TRUE );
EXTERN BOOLEAN    testdev_PNP      ( BOOLEAN seterr=TRUE );
EXTERN VOID       clean_PNP        ( VOID );


/* pnp02.cpp */
EXTERN VOID       initkey_PNP      ( VOID );
EXTERN BOOLEAN    isolate_PNP      ( INT_2 isodata=CONFIG_RESET_CSN );


/* pnp03.cpp */
EXTERN CHAR      *devid_PNP         ( UCHAR *key );
EXTERN BOOLEAN    loadcard_PNP      ( INT_2 csn );
#if 0
EXTERN BOOLEAN    setupcards_PNP    ( VOID );
#endif


/* pnp04.cpp */
EXTERN BOOLEAN    dumprcs_PNP       ( INT_2 ucsn_idx=-1 );
EXTERN BOOLEAN    dumpdev_PNP       ( INT_2 udsn_idx=-1, INT_2 ucsn_idx=-1 );

/* pnp05.cpp */
EXTERN VOID       udelay           ( long amt );
EXTERN CHAR      *binstring1       ( UCHAR val );
EXTERN CHAR      *binstring2       ( INT_2 val );
EXTERN CHAR      *binstring4       ( INT_4 val );


/*-------------------------------------------------------------------------*/
#endif


