/*  pnp01.cc : June 5, 1996  */

/* Copyright (C) 1996-1999  Sekhar Bala,
 *                          Ramchander Balasubramanian, and
 *                          Alphax Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>

#include "std.h"
#include "pnp.h"


/*-------------------------------------------------------------------------*/

GLOBAL BOOLEAN          verbose                = FALSE;
GLOBAL INT_2            PNP_port               = 0;
GLOBAL INT_2            PNP_csn                = 0;
GLOBAL INT_2            PNP_csn_idx            = -1;
GLOBAL INT_2            PNP_device             = -1;

GLOBAL INT_2            PNP_last_csn           = 1;
GLOBAL PNPCARD_TYP      PNP_cards[MAX_CARDS+1] = {  { 0, },  };
GLOBAL INT_2            PNP_cards_len          = 0;

/*-------------------------------------------------------------------------*/


GLOBAL BOOLEAN init_PNP( VOID )
{
 int            retval;
 INT_2          i;
 BOOLEAN        err     = FALSE;
 PNPCARD_TYP   *card    = NULL;

   /* clear data */
   PNP_port      = 0;
   PNP_csn       = 0;
   PNP_csn_idx   = -1;
   PNP_device    = -1;
   PNP_last_csn  = 1;
   PNP_cards_len = 0;
   for( i=0; (i < MAX_CARDS+1); i++ ) {
      card = &PNP_cards[i];
      memset( card, 0, sizeof(PNPCARD_TYP) );
   }

   /* get permission to use ports */
   retval = ioperm(ADDRESS_ADDR, 1, 1);
   if ( retval < 0 ) {
      err = TRUE;
      set_errhdl(ERR_SYS_PERM);
      goto x_done;
   }

   /* get access to WRITE_DATA port > 0x3ff */
   retval = iopl(3);
   if ( retval < 0 ) {
      err = TRUE;
      set_errhdl(ERR_SYS_PERM);
      retval = ioperm( ADDRESS_ADDR, 1, 0 );
      goto x_done;
   }

x_done:
   return( err );

} /* init_PNP */


GLOBAL BOOLEAN defaultport_PNP( INT_2 port )
{
 int  retval;

   if ( port == 0 )
      port = PNP_DATAPORT_MIN;
   if ( port == PNP_port )
      return( TRUE );
   if ( PNP_port != 0 )
      retval = ioperm( PNP_port, 1, 0 );
   PNP_port = port;
   retval = ioperm( PNP_port, 1, 1 );
   return( TRUE );

} /* defaultport_PNP */


GLOBAL VOID statuswait_PNP( VOID )
{
 UCHAR   dbyte;

   R_STATUS;
   dbyte = READ_DATA;
   while ( !(dbyte & 1) ) {
      R_STATUS;
      dbyte = READ_DATA;
   }
   return;

} /* statuswait_PNP */


GLOBAL BOOLEAN current_PNP( INT_2 ucsn )
{
 INT_2          i;
 INT_2          csn;
 PNPCARD_TYP   *card;

   csn = ucsn;
   if ( csn == -1 )
      csn = PNP_csn;
   for( i=0; (i < PNP_cards_len); i++ ) {
      card = &PNP_cards[i];
      if ( card->csn == csn ) {
         PNP_csn     = card->csn;
         PNP_csn_idx = i;
         if ( card->dev_len > 0 )
            PNP_device  = 0;
         return( FALSE );
      }
   }
   return( TRUE );

} /* current_PNP */


GLOBAL BOOLEAN testcsn_PNP( BOOLEAN seterr )
{

   if ( (PNP_csn <= 0) || (PNP_csn_idx < 0) ) {
      if ( seterr )
         set_errhdl( ERR_SYS_DEVICE );
      return( FALSE );
   }
   return( TRUE );

} /* testcsn_PNP */


GLOBAL BOOLEAN testdev_PNP( BOOLEAN seterr )
{

   if ( PNP_device < 0 ) {
      if ( seterr )
         set_errhdl( ERR_SYS_DEVICE );
      return( FALSE );
   }
   return( TRUE );

} /* testdev_PNP */


GLOBAL VOID clean_PNP( VOID )
{
 int           retval;
 INT_2         i;
 PNPCARD_TYP  *card;

   retval = ioperm( ADDRESS_ADDR, 1, 0 );
   if ( PNP_port != 0 )
      retval = ioperm( PNP_port, 1, 0 );
   for( i=0; (i < PNP_cards_len); i++ ) {
      card = &PNP_cards[i];
      if ( card->csn == 0 )
         continue;
      ::free( card->data );
      ::free( card->dev );
      memset( card, 0, sizeof(PNPCARD_TYP) );
   }
   return;

} /* clean_PNP */


/*-------------------------------------------------------------------------*/

