/*  pnp02.cc : June 5, 1996  */

/* Copyright (C) 1996-1999  Sekhar Bala,
 *                          Ramchander Balasubramanian, and
 *                          Alphax Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>

#include "std.h"
#include "pnp.h"


/*-------------------------------------------------------------------------*/

#ifdef GO32
#define LOCK_PREFIX ""
#define ADDR  (*(const struct __dummy *) addr)

int set_bit(int nr, void * addr)
{
	int oldbit;

	__asm__ __volatile__(LOCK_PREFIX
		"btsl %2,%1\n\tsbbl %0,%0"
		:"=r" (oldbit),"=m" (ADDR)
		:"ir" (nr));
	return oldbit;
}

int test_bit(int nr, const void * addr)
{
	return ((1UL << (nr & 31)) & (((const unsigned int *) addr)[nr >> 5])) != 0;
}
#endif

/*-------------------------------------------------------------------------*/


//
// testserial_PNP :: checksum serial id
//        returns :: value of checksum
//
STATIC UCHAR testserial_PNP( UCHAR *key )
{
 UCHAR   chk1, chk2;
 INT_2   loop1, loop2;

   chk1 = 0x6A;
   for( loop1=0; (loop1 <= 7); loop1++ ) {
      for( loop2=0; (loop2 <= 7); loop2++ ) {
         chk2   = (test_bit(loop2,key+loop1)) ? 0x01 : 0x00 ;
         chk2  ^= chk1;
         chk1 >>= 1;
         chk2  ^= chk1;
         chk1  &= 0x7F;
         chk1  |= (chk2 << 7) & 0x80;
      }
   }
   return( chk1 );

} /* testserial_PNP */


//
// initkey_PNP :: send ISA card initialization strings
//     returns :: none
//
GLOBAL VOID initkey_PNP( VOID )
{
 #define _DELAY               250
 #define R_INITKEY_DATALEN    32
 STATIC UCHAR  initdata[R_INITKEY_DATALEN] = {
                              0x6A, 0xB5, 0xDA, 0xED, 0xF6, 0xFB, 0x7D, 0xBE,
                              0xDF, 0x6F, 0x37, 0x1B, 0x0D, 0x86, 0xC3, 0x61,
                              0xB0, 0x58, 0x2C, 0x16, 0x8B, 0x45, 0xA2, 0xD1,
                              0xE8, 0x74, 0x3A, 0x9D, 0xCE, 0xE7, 0x73, 0x39
                              };
        INT_2  i;

   ADDRESS(0);
   udelay(_DELAY);
   ADDRESS(0);
   for( i=0; (i < R_INITKEY_DATALEN); i++ ) {
      udelay(_DELAY);
      ADDRESS( initdata[i] );
   }
   udelay(_DELAY);
   return;

 #undef R_INITKEY_DATALEN
 #undef _DELAY
} /* initkey_PNP */


//
// discover_PNP() :: attempt to discover individual card
//        returns :: TRUE if valid card found, else FALSE
//
STATIC BOOLEAN discover_PNP( VOID )
{
 INT_2          loop1, loop2;
 UCHAR          ch1,   ch2;
 INT_2          csn;
 BOOLEAN        found;
 UCHAR          inbuf[SERIAL_LEN];
 PNPCARD_TYP   *card;

   // send wake(0) to put all undiscovered cards in 'isolation' state
   R_WAKE(0x00);

   // send set read data port
   R_SETRDPORT( PNP_port );

   // read the serial key
   R_SERIALISOLATION;
   udelay(750);
   found = FALSE;
   for( loop1=0; (loop1 < SERIAL_LEN); loop1++ ) {
      inbuf[loop1] = 0;
      for( loop2=0; (loop2 <= 7); loop2++ ) {
         // read mext pair 0x????->0bit 0x55AA->1bit
         udelay(250);
         ch1 = READ_DATA;
         ch2 = READ_DATA;
         // special tests
         if ( (ch1 == 0x55) && (ch2 == 0xAA) ) {
            set_bit(loop2, inbuf+loop1);
            found = TRUE;
         }
      }
   }

   // make sure it was correct
   if ( (!found) || (testserial_PNP(inbuf) != inbuf[8]) ) {
      if ( verbose ) {
         printf( "CHK: P=%04X ", PNP_port );
         printf( "%s ",          devid_PNP(inbuf) );
         printf( "%02X ",        inbuf[8] );
         printf( "\n" );
      }
      return( FALSE );
   }

   // send a CSN to the card
   csn = PNP_last_csn++;
   R_CARDSELECTNUMBER;
   WRITE_DATA(csn);

   // setup info
   card = &PNP_cards[PNP_cards_len++];
   card->csn = csn;
   memcpy( card->serial, inbuf, SERIAL_LEN );
   strcpy( card->id, devid_PNP(card->serial) );
   card->unique = *(INT_U2 *) (card->serial+4);

   // done
   return( TRUE );

} /* discover_PNP() */


//
// isolate_PNP() :: Discover and initialize the PnP cards in the system
//       returns :: TRUE if valid card found, else FALSE
//                  NOTE: 0x223 looks like the power-up-default of 1869 card
//
GLOBAL BOOLEAN isolate_PNP( INT_2 isodata )
{
 int        retval;
 INT_U2     port_ok = 0;

   // determine the read port to start on and release any permissions
   if ( PNP_port != 0 )
      retval = ioperm( PNP_port, 1, 0 );
   PNP_port = PNP_DATAPORT_MIN;

   // put all the cards into the sleep state
   initkey_PNP();

   // reset all CSNs
   R_CONFIGCONTROL;
   WRITE_DATA(isodata);
   if ( isodata != CONFIG_RESET_CSN )
      udelay(750);

   // step through the port range looking for a good port
   while ( TRUE ) {

      // test max
      if ( (PNP_port >= PNP_DATAPORT_MAX) || (PNP_cards_len >= MAX_CARDS+1) )
         break;

      // make sure we have the io permissions
      retval = ioperm( PNP_port, 1, 1 );

      // try to discover each card individually on the current port
      if ( discover_PNP() ) {
          // remember port if it worked
          port_ok = PNP_port;
          continue;
      }

      // try a different port
      if ( port_ok != 0 )
         break;

      // release permission and bump next port
      retval = ioperm( PNP_port, 1, 0 );
      PNP_port += 8;

   } /* endwhile */

   // return cards to wait-for-key state
   R_CONFIGCONTROL;
   WRITE_DATA(CONFIG_WAIT4KEY);

   // indicate whether any cards were found
   if ( port_ok == 0 )
      return( FALSE );

   // done
   PNP_port = port_ok;
   return( TRUE );

} /* isolate_PNP */


/*-------------------------------------------------------------------------*/

