/*  pnp03.cc : June 5, 1996  */

/* Copyright (C) 1996-1999  Sekhar Bala,
 *                          Ramchander Balasubramanian, and
 *                          Alphax Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>

#include "std.h"
#include "pnp.h"


/*-------------------------------------------------------------------------*/

#define MAX_BUFSIZE  (4 * 1024)

STATIC BOOLEAN    isacard_prepared     = FALSE;

STATIC INT_2      buf_len              = 0;
STATIC UCHAR      buf[MAX_BUFSIZE]     = { 0, };

/*-------------------------------------------------------------------------*/


//
// devid_PNP :: format serial number
//   returns :: ptr to static string
//
GLOBAL CHAR *devid_PNP( UCHAR *key )
{
 STATIC CHAR      resstd[10] = { "PNP0000" };

   sprintf( resstd, "%c%c%c%x%x%x%x",
            'A' + (key[0] >> 2) - 1,
            'A' + (((key[0] & 3) << 3) | (key[1] >> 5)) - 1,
            'A' + (key[1] & 0x1F) -1,
            key[2] >> 4,
            key[2] & 0x0F,
            key[3] >> 4,
            key[3] & 0x0F );
   return( resstd );

} /* devid_PNP */


//
// getresdata_PNP()  :: get next data byte of resource info
//          returns  :: returns data or ok set FALSE
//
STATIC UCHAR getresdata_PNP( BOOLEAN &ok )
{
 INT_4   loop;
 UCHAR   dbyte;

   if ( !ok )
      return( FALSE );
   for( loop=32767; (loop > 0); loop-- ) {
      R_STATUS;
      dbyte = READ_DATA;
      if ( dbyte & 0x01 )
         break;
   }
   if ( loop <= 0 ) {
      ok = FALSE;
      return( FALSE );
   }

   R_RESOURCEDATA;
   dbyte = READ_DATA;
   return( dbyte );

} /* getresdata_PNP */


//
// loadresource_PNP() :: setup the card info without 'isolation'
//            returns :: TRUE if valid card found, else FALSE
//
STATIC BOOLEAN loadresource_PNP( INT_2 csn_idx )
{
 INT_2          i;
 INT_2          j;
 INT_2          tmp;
 PNPCARD_TYP   *card;
 PNPDEV_TYP    *dev;
 UCHAR         *ptr;
 UCHAR         *devptr  = NULL;
 BOOLEAN        same    = TRUE;
 BOOLEAN        ok      = TRUE;

   // setup
   card = &PNP_cards[csn_idx];
   R_WAKE(card->csn);

   // setup data
   for( i=0; ((i < 9) && ok); i++ ) {
      card->serial[i] = getresdata_PNP(ok);
      if ( i != 0 ) {
         if ( card->serial[i] != card->serial[i-1] )
            same = FALSE;
      }
   }
   if ( same )
      return( FALSE );

   // get large/small descriptors
   buf_len = 0;
   ptr     = buf;
   while (TRUE) {

      // test for termination
      if ( (!ok) || (buf_len >= MAX_BUFSIZE) )
         break;

      // get an initiating byte
      *ptr = getresdata_PNP(ok);

      // test & transfer large descriptors
      if ( (*ptr & longmask_TAG) != 0 ) {
         ptr++;
         *ptr++   = getresdata_PNP(ok);
         *ptr++   = getresdata_PNP(ok);
         buf_len += 3;
         tmp      = *(INT_2 *)(ptr-2);
         if ( (!ok) || (buf_len+tmp > MAX_BUFSIZE) )
            break;
         for( i=0; (i < tmp); i++ ) {
            if ( (!ok) || (buf_len >= MAX_BUFSIZE) )
               break;
            *ptr = getresdata_PNP(ok);
            ptr++;
            buf_len++;
         }
         if ( (!ok) || (buf_len >= MAX_BUFSIZE) )
            break;
         continue;
      }

      // else short tag, stop when end tag is found
      if ( _short_TAG(*ptr,end_TAG) ) {
         ptr++;
         buf_len++;
         break;
      }

      // update log_dev count
      devptr = NULL;
      if ( _short_TAG(*ptr, logdevid_TAG) ) {
         devptr = ptr;
         tmp    = sizeof(PNPDEV_TYP) * (card->dev_len+1);
         if ( card->dev_len == 0 ) {
            card->dev = (PNPDEV_TYP (*)[]) ::malloc( tmp );
         } else {
            card->dev = (PNPDEV_TYP (*)[]) ::realloc( card->dev, tmp );
         }
         if ( card->dev == NULL ) {
            set_errhdl( ERR_SYS_NOMEM );
            return( FALSE );
         }
         memset( &(*card->dev)[card->dev_len], 0, sizeof(PNPDEV_TYP) );
         for( j=0; (j < DL_DMA_LEN); j++ )
            (*card->dev)[card->dev_len].dma[j] = 0x04;
      }

      // transfer other small descriptors
      tmp = (INT_2) *ptr & 0x07;
      ptr++;
      buf_len++;
      for( i=0; (i < tmp); i++ ) {
         if ( (!ok) || (buf_len >= MAX_BUFSIZE) )
            break;
         *ptr = getresdata_PNP(ok);
         ptr++;
         buf_len++;
      }
      if ( devptr != NULL ) {
         dev      = &(*card->dev)[card->dev_len];
         dev->dsn = card->dev_len;
         strcpy( dev->id, devid_PNP(&devptr[1]) );
         card->dev_len++;
      }
      if ( (!ok) || (buf_len >= MAX_BUFSIZE) )
         break;

   } /* endwhile */

   if ( !ok )
      return( FALSE );

   // continue data transfer
   strcpy( card->id, devid_PNP(card->serial) );
   card->unique   = *(INT_U2 *)(card->serial+4);
   card->data_len = buf_len;
   card->data     = (UCHAR *) ::malloc( card->data_len );
   if ( card->data == NULL ) {
      set_errhdl( ERR_SYS_NOMEM );
   } else {
      memcpy( card->data, buf, card->data_len );
   }

   // done
   return( TRUE );

} /* loadresource_PNP */


//
//  readINTU2 :: Simple INT_2 address reader
//    returns :: INT_U2 data in normal reverse form
//
//
STATIC INT_U2 readINTU2( INT_2 addr )
{
 INT_U2  retval;
 UCHAR   tmp[2];

   ADDRESS(addr+0);
   tmp[0] = READ_DATA;
   ADDRESS(addr+1);
   tmp[1] = READ_DATA;
   retval = ((INT_U2) tmp[0] << 8) | ((INT_U2) tmp[1] & 0x00FF);
   return( retval );

} /* readINTU2 */


//
// loadregisters_PNP() :: load card cfg without 'isolation'
//             returns :: TRUE if valid card found, else FALSE
//
STATIC BOOLEAN loadregisters_PNP( INT_2 csn_idx )
{
 INT_2          i;
 INT_2          tmp;
 INT_U2         low;
 INT_U2         high;
 PNPCARD_TYP   *card;
 PNPDEV_TYP    *dev;

   // setup
   card = &PNP_cards[csn_idx];
   R_WAKE(card->csn);

   // get/save vender defined card level registers
   for( i=CL_VENDOR_BASE; (i <= CL_VENDOR_LIMIT); i++ ) {
      ADDRESS(i);
      card->vendor[ (i-CL_VENDOR_BASE) ] = READ_DATA;
   }

   // get/save registers for each logical device
   for( i=0; (i < card->dev_len); i++ ) {

      // setup
      dev = &(*card->dev)[i];

      // select device
      R_LOGICALDEVICENUMBER;
      WRITE_DATA(dev->dsn);
#if 0
      // this should always work, in theory
      if ( READ_DATA != dev->dsn ) {
         // ( "Cannot select device %d\n", i );
         continue;
      }
#endif
      R_ACTIVATE;
      dev->active = READ_DATA;

      R_IORANGECHECK;
      dev->rangecheck = READ_DATA;

      // MEM 8/16 bits
      for( i=DL_MEM_BASE; (i <= DL_MEM_LIMIT); i += DL_MEM_INC ) {
         tmp    = (i - DL_MEM_BASE) / DL_MEM_INC ;
         dev->mem[ tmp ].base  = readINTU2( i );
         ADDRESS(i+2);
         dev->mem[ tmp ].flag  = READ_DATA;
         dev->mem[ tmp ].limit = readINTU2( i+3 );
      }

      // MEM 32 bits
      for( i=DL_MEM32_BASE; (i <= DL_MEM32_LIMIT); i += DL_MEM32_INC ) {
         tmp    = (i - DL_MEM32_BASE) / DL_MEM32_INC ;
         if ( i == DL_MEM32_BASE )
            i += 6;
         high = readINTU2( i+0 );
         low  = readINTU2( i+2 );
         dev->mem32[ tmp ].base  = (((INT_U4) high << 16) | low);
         ADDRESS( i+4 );
         dev->mem32[ tmp ].flag  = READ_DATA;
         high = readINTU2( i+5 );
         low  = readINTU2( i+7 );
         dev->mem32[ tmp ].limit = (((INT_U4) high << 16) | low);
         if ( i == DL_MEM32_BASE+6 )
            i -= 6;
      } /* endfor */

      // I/O Port
      for( i=DL_IO_BASE; (i <= DL_IO_LIMIT); i += DL_IO_INC ) {
         tmp = (i - DL_IO_BASE) / DL_IO_INC;
         dev->ioport[ tmp ] = readINTU2( i );
      }

      // IRQ
      for( i=DL_IRQ_BASE; (i <= DL_IRQ_LIMIT); i += DL_IRQ_INC ) {
         tmp = (i - DL_IRQ_BASE) / DL_IRQ_INC;
         ADDRESS(i+0);
         dev->irq[ tmp ][0] = READ_DATA;
         ADDRESS(i+1);
         dev->irq[ tmp ][1] = READ_DATA;
      }

      // dma0 & dma1
      for( i=DL_DMA_BASE; (i <= DL_DMA_LIMIT); i += DL_DMA_INC ) {
         tmp = (i - DL_DMA_BASE) / DL_DMA_INC;
         ADDRESS(i);
         dev->dma[ tmp ] = READ_DATA;
      }

      // vendor defined
      for( i=DL_VENDOR_BASE; (i <= DL_VENDOR_LIMIT); i += DL_DMA_INC ) {
         tmp = (i - DL_VENDOR_BASE) / DL_DMA_INC;
         ADDRESS(i);
         dev->vendor[ tmp ] = READ_DATA;
      }

   } /* endfor */

   // done
   return( TRUE );

} /* loadregisters_PNP */


//
// loadcard_PNP() :: setup the card info without 'isolation'
//        returns :: TRUE if valid card found, else FALSE
//
GLOBAL BOOLEAN loadcard_PNP( INT_2 csn )
{
 INT_2           csn_idx;
 PNPCARD_TYP    *card   = NULL;
 BOOLEAN         ok     = TRUE;

   // initialize
   for( csn_idx=0; (csn_idx < PNP_cards_len); csn_idx++ ) {
       card = &PNP_cards[csn_idx];
       if ( card->csn == 0 )
          continue;
       if ( card->csn == csn ) {
          if ( card->data != NULL )
             ::free( card->data );
          if ( card->dev != NULL )
             ::free( card->dev );
          memset( card, 0, sizeof(PNPCARD_TYP) );
          break;
       }
   }
   if ( csn_idx == PNP_cards_len )
      card = &PNP_cards[PNP_cards_len++];

   // make sure isacards are prepared
   card->csn = csn;
   if ( !isacard_prepared ) {
      initkey_PNP();
      R_SETRDPORT( PNP_port );
   }

   if ( !loadresource_PNP(csn_idx) ) {
      ok = FALSE;
      goto x_done;
   }

   if ( !loadregisters_PNP(csn_idx) ) {
      ok = FALSE;
      // goto x_done;
   }

x_done:

   // make sure isacards are 'un'prepared
   if ( !isacard_prepared ) {
      R_CONFIGCONTROL;
      WRITE_DATA( CONFIG_WAIT4KEY );
   }

   // done
   return( ok );

} /* loadcard_PNP */


#if 0
//
// setupcards_PNP() :: setup info for all known cards without 'isolation'
//          returns :: TRUE if valid cards found, else FALSE
//
GLOBAL BOOLEAN setupcards_PNP( VOID )
{
 INT_2    i;
 BOOLEAN  ok = TRUE;

   // hardware setup
   initkey_PNP();
   R_SETRDPORT( PNP_port );
   isacard_prepared = TRUE;

   // for every isolated card, load the resource info
   for( i=0; (i < PNP_cards_len); i++ ) {
      if ( PNP_cards[i].csn == 0 )
         continue;
      if ( !loadcard_PNP(PNP_cards[i].csn) )
         ok = FALSE;
   }

   // hardware cleanup
   isacard_prepared = FALSE;
   R_CONFIGCONTROL;
   WRITE_DATA( CONFIG_WAIT4KEY );

   // done
   return( ok );

} /* setupcards_PNP */
#endif


/*-------------------------------------------------------------------------*/


