/*  pnp04.cc : June 5, 1996  */

/* Copyright (C) 1996-1999  Sekhar Bala,
 *                          Ramchander Balasubramanian, and
 *                          Alphax Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>

#include "std.h"
#include "parse.h"
#include "input.h"
#include "pnp.h"


/*--------------------------------------------------------------------------*/

typedef struct {
   INT_2       code;
   CHAR       *name;
} XTAGDESC_TYP;

#define  _MARK(x)    x, #x
STATIC XTAGDESC_TYP    stags[] = {
   { _MARK(sunknown_TAG)         },
   { _MARK(pnpverno_TAG)         },
   { _MARK(logdevid_TAG)         },
   { _MARK(cmptdevid_TAG)        },
   { _MARK(irq_TAG)              },
   { _MARK(dma_TAG)              },
   { _MARK(startdep_TAG)         },
   { _MARK(enddep_TAG)           },
   { _MARK(ioport_TAG)           },
   { _MARK(fixedio_TAG)          },
   { _MARK(rsvdA_TAG)            },
   { _MARK(rsvdB_TAG)            },
   { _MARK(rsvdC_TAG)            },
   { _MARK(rsvdD_TAG)            },
   { _MARK(svendor_TAG)          },
   { _MARK(end_TAG)              },
   { 0, NULL }
};

STATIC XTAGDESC_TYP    ltags[] = {
   { _MARK(lunknown_TAG)         },
   { _MARK(mem_TAG)              },
   { _MARK(ansistr_TAG)          },
   { _MARK(unicodestr_TAG)       },
   { _MARK(lvendor_TAG)          },
   { _MARK(mem32_TAG)            },
   { _MARK(fixedmem32_TAG)       },
   { 0, NULL }
};
#undef _MARK

STATIC CHAR *irqtags[] = { "ld",  "ll", "he", "hl" };

#define mkINT4(v,p)           \
    p++;                      \
    v  = (INT_4) *p & 0xFF;   \
    p++;                      \
    v += ((INT_4) *p <<  8);  \
    p++;                      \
    v += ((INT_4) *p << 16);  \
    p++;                      \
    v += ((INT_4) *p << 24);  \
    p++

#define mkINT2(v,p)           \
    p++;                      \
    v  = (INT_2) *p & 0xFF;   \
    p++;                      \
    v += ((INT_2) *p <<  8);  \
    p++

STATIC BOOLEAN      dashed = FALSE;

/*--------------------------------------------------------------------------*/


STATIC VOID parse_shortTAG( UCHAR **data, INT_2 *data_len )
{
 UCHAR   *ptr;
 INT_2    ptr_len;
 INT_2    i;
 UCHAR    tmp1;
 INT_2    tmp2;
 UCHAR    ddata;
 INT_2    dlen;
 CHAR    *text;

   // setup
   ptr     = (*data);
   ptr_len = (*data_len);
   ddata   = (*ptr) & shortmask_TAG;
   dlen    = ((INT_2) *ptr) & 0x07;

   // search for match
   for( i=0; (TRUE); i++ ) {
      if ( stags[i].name == NULL ) {
         i = 0;
         break;
      }   
      if ( ddata == stags[i].code )
         break;
   }

   if ( (!dashed) && (ddata == logdevid_TAG) ) {
      fprintf( outfp, "  ---------------\n" );
      dashed = TRUE;
   } else {
      dashed = FALSE;
   }
   fprintf( outfp, "  %-15.15s  ", stags[i].name );
   if ( verbose )
      fprintf( outfp, "(x%02X) [%3d] ", (ddata >> 3), dlen );
   switch( ddata ) {
      case pnpverno_TAG:
         ptr++;
         fprintf( outfp, "%d.%d", (*ptr >> 4), (*ptr & 0x0F) );
         ptr++;
         if ( dlen > 1 )
            fprintf( outfp, " - x%02X", *ptr );
         break;
      case logdevid_TAG:
         ptr++;
         text = devid_PNP( ptr );
         fprintf( outfp, "%s ", text );
         ptr += 2;
         mkINT2(tmp2,ptr);
         text = binstring2( tmp2 );
         fprintf( outfp, " %s (0x3F-31 0=bootable)", text );
         break;
      case cmptdevid_TAG:
         ptr++;
         text = devid_PNP( ptr );
         fprintf( outfp, "%s ", text );
         break;
      case irq_TAG:
         mkINT2(tmp2,ptr);
         fprintf( outfp, "( " );
         for( i=0; (i < 16); i++ ) {
            if ( (tmp2 & 0x01) != 0 )
               fprintf( outfp, "%d ", i);
            tmp2 >>= 1;
         }
         fprintf( outfp, ")");
         if ( dlen > 2 ) {
            text = binstring1( *ptr );
            fprintf( outfp, " Sense: %s", text );                   // ?? more
         }
         break;
      case dma_TAG:
         ptr++;
         fprintf( outfp, "( " );
         tmp1 = *ptr;
         for( i=0; (i < 8); i++ ) {
            if ( (tmp1 & 0x01) != 0 )
               fprintf( outfp, "%d ", i);
            tmp1 >>= 1;
         }
         fprintf( outfp, ")" );
         ptr++;
         text = binstring1( *ptr );
         fprintf( outfp, " flg: %s", text );                        // ?? more
         break;
      case startdep_TAG:
         ptr++;
         if ( dlen > 0 ) {
            switch( *ptr ) {
               case 0:
                  fprintf( outfp, "Good Cfg" );
                  break;
               case 1:
                  fprintf( outfp, "Acceptable Cfg" );
                  break;
               case 2:
                  fprintf( outfp, "Sub-optimal Cfg" );
                  break;
               default:
                  fprintf( outfp, "Reserved" );
                  break;
            }
         }
         break;
      case enddep_TAG:
         // no data just formatting
         fprintf( outfp, "\n" );
         fprintf( outfp, "  --------------- " );
         dashed = TRUE;
         break;
      case ioport_TAG:
         ptr++;
         tmp2 = 10;
         if ( (*ptr & 0x01) != 0 )
            tmp2 += 6;
         fprintf( outfp, "%d bits", tmp2 );
         mkINT2(tmp2,ptr);
         fprintf( outfp, " x%04X-", tmp2 );
         ptr--;
         mkINT2(tmp2,ptr);
         fprintf( outfp, "x%04X", tmp2 );
         fprintf( outfp, " Align: x%02X", *ptr );
         ptr++;
         fprintf( outfp, " Len: %d", ((INT_2) *ptr & 0xFF) );
         break;
      case fixedio_TAG:
         mkINT2(tmp2,ptr);
         fprintf( outfp, "10 bits x%04X", tmp2 );
         ptr++;
         fprintf( outfp, " Len: %d", ((INT_2) *ptr & 0xFF) );
         break;
      case svendor_TAG:
         ptr++;
         for( i=0; (i < dlen); i++ ) {
            fprintf( outfp, "[x%02X] ", *ptr++ );
         }
         break;
      case end_TAG:
         ptr++;
         fprintf( outfp, "checksum: x%02X", *ptr);
         ptr_len = 0;
         goto x_done;
      default:
         break;
   }

   dlen    += 1;
   ptr      = (*data) + dlen;
   ptr_len -= dlen;

x_done:

   // cleanup
   fprintf( outfp, "\n" );
   (*data)     = ptr;
   (*data_len) = ptr_len;

   // done
   return;

} /* parse_shortTAG */


STATIC VOID parse_longTAG( UCHAR **data, INT_2 *data_len )
{
 UCHAR  *ptr;
 INT_2   ptr_len;
 INT_2   i;
 UCHAR   ddata;
 INT_2   dlen;

   // setup
   ptr     = (*data);
   ptr_len = (*data_len);
   ddata   = *ptr & 0x7F;
   mkINT2(dlen,ptr);

   if ( ddata <= fixedmem32_TAG ) {
      fprintf( outfp, "  %-15.15s* ", ltags[ddata].name );
      if ( verbose )
         fprintf( outfp, "[%04X-%d] ", ddata, dlen );
      switch( ddata ) {
         case ansistr_TAG:
            fprintf( outfp, "\"" );
            for( i=0; (i < dlen); i++ ) {
               fprintf( outfp, "%c", *ptr++ );
            }
            fprintf( outfp, "\"" );
            break;
         default:
            break;
      }
      goto x_clean;
   }

   fprintf( outfp, "  %-15.15s* ", "lrsvd_TAG" );
   if ( verbose )
      fprintf( outfp, " [x%04X-%d]", ddata, dlen );

x_clean:

   dlen    += 3;
   ptr      = (*data) + dlen;
   ptr_len -= dlen;

   // cleanup
   fprintf( outfp, "\n" );
   (*data)     = ptr;
   (*data_len) = ptr_len;

   // done
   return;

} /* parse_longTAG */


GLOBAL BOOLEAN dumprcs_PNP( INT_2 ucsn_idx )
{
 INT_2             i;
 INT_2             csn_idx;
 PNPCARD_TYP      *card;
 UCHAR            *serial;
 UCHAR            *ptr;
 INT_2             ptr_len;

   // determine ISA card
   csn_idx = ucsn_idx;
   if ( csn_idx == -1 )
      csn_idx = PNP_csn_idx;
   if ( (csn_idx < 0) || (csn_idx >= PNP_cards_len) )
      goto x_err;

   // setup
   card = &PNP_cards[csn_idx];
   if ( card->csn <= 0 )
      goto x_err;

   // header
   fprintf( outfp, "card[%02d]<%03d>: ", card->csn, card->dev_len );

   // setup
   serial  = card->serial;
   ptr     = card->data;
   ptr_len = card->data_len;

   // serial number
   fprintf( outfp, "%s x%04X",  card->id, card->unique );
   if ( verbose ) {
      fprintf( outfp, " -> 0x%08X%08X",
                      *(INT_U2 *)(serial), *(INT_U2 *)(serial+4) );
      fprintf( outfp, " (d=%d)", card->data_len );
      fprintf( outfp, "\n" );
      fprintf( outfp, "                " );
      for( i=0; (i < CL_VENDOR_LEN); i++ )
            fprintf( outfp, "%02X ", card->vendor[i] );
   }
   fprintf( outfp, "\n" );

   // leaf thru the data
   dashed = TRUE;
   fprintf( outfp, "  ---------------\n" );
   while ( ptr_len > 0 ) {

      // test for long descriptor
      if ( (*ptr & longmask_TAG) != 0 ) {
         parse_longTAG( &ptr, &ptr_len );
         continue;
      }

      // else short tag
      parse_shortTAG( &ptr, &ptr_len );

   } /* endwhile */
   if ( !dashed )
      fprintf( outfp, "  ---------------\n" );

   // done
   return( FALSE );

x_err:

   set_errhdl( ERR_SYS_DEVICE );
   return( TRUE );

} /* dumprsc_PNP */


GLOBAL BOOLEAN dumpdev_PNP( INT_2 udsn_idx, INT_2 ucsn_idx )
{
 INT_2          i;
 INT_2          dsn_idx;
 INT_2          csn_idx;
 INT_2          tmp;
 BOOLEAN        found;
 BOOLEAN        title;
 PNPCARD_TYP   *card;
 PNPDEV_TYP    *dev;

   // determine ISA card
   csn_idx = ucsn_idx;
   if ( csn_idx < 0 )
      csn_idx = PNP_csn_idx;
   if ( (csn_idx < 0) || (csn_idx >= PNP_cards_len) )
      goto x_err;

   // setup
   card = &PNP_cards[csn_idx];
   if ( card->csn <= 0 )
      goto x_err;

   // determine Device on card
   dsn_idx = udsn_idx;
   if ( dsn_idx < 0 )
      dsn_idx = PNP_device;
   if ( (dsn_idx < 0) || (dsn_idx >= card->dev_len) )
      goto x_err;
   if ( card->dev == NULL )
      goto x_err;
   dev = &(*card->dev)[dsn_idx];

   // header
   fprintf( outfp, "  ===========\n" );
   fprintf( outfp, "  Device %03d: %s ", dsn_idx, dev->id );
   if ( (dev->active & 0x01) != 0 )
      fprintf( outfp, "(ACTIVE" );
   else
      fprintf( outfp, "(INACTIVE" );
   if ( verbose )
      fprintf( outfp, "-x%02X", dev->active );
   fprintf( outfp, ") " );
   if ( verbose )
      fprintf( outfp, "RANGECHECK=x%02X", dev->rangecheck );
   fprintf( outfp, "\n" );

   // mem
   found = FALSE;
   title = FALSE;
   for( i=0; (i < DL_MEM_LEN); i++ ) {
      if ( dev->mem[i].base == 0 )
         continue;
      if ( !title ) {
         fprintf( outfp, "         MEM: " );
         title = TRUE;
      }
      fprintf( outfp, "x%04X ", dev->mem[i].base );
      if ( (dev->mem[i].flag & 0x01) != 0 )
         fprintf( outfp, "...." );
      else
         fprintf( outfp, "size" );
      fprintf( outfp, " x%04X ", dev->mem[i].limit );
      tmp = 8;
      if ( (dev->mem[i].flag & 0x02) == 2 )
         tmp = 16;
      fprintf( outfp, "(%02d-bit) ", tmp );
      found = TRUE;
   }
   if ( (!found) && (verbose) )
      fprintf( outfp, "         MEM: none\n" );
   if ( found )
      fprintf( outfp, "\n" );

   // mem32
   found = FALSE;
   title = FALSE;
   for( i=0; (i < DL_MEM32_LEN); i++ ) {
      if ( dev->mem32[i].base == 0 )
         continue;
      if ( !title ) {
         fprintf( outfp, "       MEM32: " );
         title = TRUE;
      }
      fprintf( outfp, "x%08lX ", dev->mem32[i].base );
      if ( (dev->mem32[i].flag & 0x01) != 0 )
         fprintf( outfp, "...." );
      else
         fprintf( outfp, "size" );
      fprintf( outfp, " x%08lX ", dev->mem32[i].limit );
      tmp = 8;
      if ( (dev->mem32[i].flag & 0x06) == 2 )
         tmp = 16;
      if ( (dev->mem32[i].flag & 0x06) == 6 )
         tmp = 32;
      fprintf( outfp, "(%02d-bit) ", tmp );
      found = TRUE;
   }
   if ( (!found) && (verbose) )
      fprintf( outfp, "       MEM32: none\n" );
   if ( found )
      fprintf( outfp, "\n" );

   // I/O port
   found = FALSE;
   title = FALSE;
   for( i=0; (i < DL_IO_LEN); i++ ) {
      if ( dev->ioport[i] == 0 )
         continue;
      if ( !title ) {
         fprintf( outfp, "    I/O Port: " );
         title = TRUE;
      }
      fprintf( outfp, "0x%04X ", dev->ioport[i] );
      found = TRUE;
   }
   if ( (!found) && (verbose) )
      fprintf( outfp, "    I/O Port: none\n" );
   if ( found )
      fprintf( outfp, "\n" );

   // irq
   found = FALSE;
   title = FALSE;
   for( i=0; (i < DL_IRQ_LEN); i++ ) {
      if ( dev->irq[i][0] == 0 )
         continue;
      if ( !title ) {
         fprintf( outfp, "         IRQ: " );
         title = TRUE;
      }
      fprintf( outfp, "%d (%s)",
                      dev->irq[i][0], irqtags[(dev->irq[i][1] & 0x03)] );
      found = TRUE;
   }
   if ( (!found) && (verbose) )
      fprintf( outfp, "         IRQ: none\n" );
   if ( found )
      fprintf( outfp, "\n" );

   // dma
   for( i=0; (i < DL_DMA_LEN); i++ ) {
      if ( dev->dma[i] == 0x04 ) {
         if ( verbose )
            fprintf( outfp, "        DMA%d: none\n", i );
         continue;
      }
      fprintf( outfp, "        DMA%d: %d\n", i, dev->dma[i] );
   }

   // vendor registers
   if ( verbose ) {
      fprintf( outfp, "      Vendor: " );
      for( i=0; (i < DL_VENDOR_LEN); i++ ) {
         fprintf( outfp, "%02X ", dev->vendor[i] );
      }
      fprintf( outfp, "\n" );
   }

   // tailer
   fprintf( outfp, "  ===========\n" );

   // done
   return( FALSE );

x_err:

   set_errhdl( ERR_SYS_DEVICE );
   return( TRUE );

} /* dumpdev_PNP */


/*--------------------------------------------------------------------------*/


