/*  user0.cc : June 5, 1996  */

/* Copyright (C) 1996-1999  Sekhar Bala,
 *                          Ramchander Balasubramanian, and
 *                          Alphax Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>

#include "std.h"
#include "wildcard.h"
#include "parse.h"
#include "symtab.h"
#include "input.h"

#include "main.h"
#include "pnp.h"

/*-------------------------------------------------------------------------*/


STATIC VOID setup_HLP( VOID )
{

   set_sum_help( "isolate <arg>            : perform card isolation"      );
   set_sum_help( "detect <csn-#>           : detect all card information" );
   set_sum_help( "status [csn-#]           : show status of current cfg"  );
   set_sum_help( "dump [args]              : dump current card"           );
   set_sum_help( "device <action> [values] : set device attribute"        );


   set_desc_help( "ISOLATE",
                  "FMT: ISOLATE [arg]\n"
                  "     args are:\n"
                  "       CSN       - reset card select number (*default)\n"
                  "       RESET     - reset card to factory setting\n"
                  "       RESET_DRV - reset card to power-up defaults\n"
                  );
   set_desc_help( "DETECT",
                  "FMT: DETECT [csn-#]\n"
                  "     - get information from card with csn-#\n"
                  );
   set_desc_help( "STATUS",
                  "FMT: STATUS\n"
                  "     - display status on all available info\n"
                  );
   set_desc_help( "DUMP",
                  "FMT: DUMP [args]\n"
                  "     - display information about PNP_device card\n"
                  "       args are:\n"
                  "         RESOURCES, DEVICE, DEVICE ALL, CARD(*)\n" 
                  "\n"
                  "Examples:\n"
                  "  DUMP RESOURCES\n"
                  "  DUMP DEVICE\n"
                  "  DUMP DEVICE ALL\n"
                  "  DUMP CARD\n"
                  );
   set_desc_help( "DEVICE",
                  "FMT: DEVICE <action> [values...]\n"
                  "     - available 'actions's are:\n"
                  "       BEGIN - initiate device configuration\n"
                  "       END   - terminate device configuration\n"
                  "       SET   - configure a device resource as follows:\n"
                  "             <resource> CLEAR\n"
                  "               - any keyword below + CLEAR will remove it\n"
                  "             MEM  n #1 #2 [b]\n"
                  "               - set memory mapped region n (0 - 3)\n"
                  "                 #1 = base address bits 23-8\n"
                  "                 #2 = length bits 23-7\n"
                  "                 b  = kw 8bit(*) or 16bit for mem-type\n"
                  "             MEM32 n #1 #2\n"
                  "               - set memory mapped usage\n"
                  "             IOPORT  # [, # ...]\n"
                  "               - set ioport resources\n"
                  "             IRQ n  #  [HIGH | LOW(*)]  [LEVEL | EDGE(*)]\n"
                  "               - set IRQ n (0 or 1) with #\n"
                  "             DMA n  #\n"
                  "               - set DMA n (0 or 1) #\n"
                  "             ACTIVE or INACTIVE\n"
                  "               - activate or deactivate card\n"
                  );

   return;

} /* setup_help */


GLOBAL BOOLEAN init_APP( VOID )
{
 SYMBOL_TYP   sym;

   // general
   init_PNP();
   setup_HLP();

   // internal setup
   defaultport_PNP();
   Uport          = PNP_port;

   // internal vars
   sym.type       = ST_TYPE_INT;
   if ( WC::strxfer(sym.symbol, "PNP_PORT") != 0 )
      return( FALSE );
   sym.value      = (VOID *) &Uport;
   sym.value_len  = sizeof(INT_2);
   sym.value_free = FALSE;
   symtab.insert( &sym );
   //
   sym.type       = ST_TYPE_INT;
   if ( WC::strxfer(sym.symbol, "VERBOSE") != 0 )
      return( FALSE );
   sym.value      = (VOID *) &verbose;
   sym.value_len  = sizeof(BOOLEAN);
   sym.value_free = FALSE;
   symtab.insert( &sym );
   //
   sym.type       = ST_TYPE_INT;
   if ( WC::strxfer(sym.symbol, "PNP_CSN") != 0 )
      return( FALSE );
   sym.value      = (VOID *) &PNP_csn;
   sym.value_len  = sizeof(INT_2);
   sym.value_free = FALSE;
   symtab.insert( &sym );
   //
   sym.type       = ST_TYPE_INT;
   if ( WC::strxfer(sym.symbol, "PNP_DEVICE") != 0 )
      return( FALSE );
   sym.value      = (VOID *) &PNP_device;
   sym.value_len  = sizeof(INT_2);
   sym.value_free = FALSE;
   symtab.insert( &sym );

   // done
   return( TRUE );

} /* init_APP */


GLOBAL VOID clean_APP( VOID )
{

   clean_PNP();
   end_DEVICE();
   return;

} /* clean_APP */


/*-------------------------------------------------------------------------*/


