/*  user1.cc : June 5, 1996  */

/* Copyright (C) 1996-1999  Sekhar Bala,
 *                          Ramchander Balasubramanian, and
 *                          Alphax Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>

#include "std.h"
#include "wildcard.h"
#include "parse.h"
#include "symtab.h"
#include "input.h"

#include "main.h"
#include "pnp.h"

/*-------------------------------------------------------------------------*/

GLOBAL INT_2      Uport;

/*-------------------------------------------------------------------------*/


STATIC BOOLEAN getINT( INT_4 *value, BOOLEAN empty=FALSE )
{
 BOOLEAN  result;

   result = eval_expr(value);
   if ( !empty ) {
      if (!result)
         return( FALSE );
      return(TRUE);
   }
   clr_errhdl();
   return( TRUE );

} /* getINT */


GLOBAL BOOLEAN exec_isolate( VOID )
{
 INT_2  isodata;

   // isolate
   if ( strcmpi(P.word, "ISOLATE") != 0 )
      return( FALSE );

   // set arg
   isodata = CONFIG_RESET_CSN;
   if ( !P.get() ) {
      if        ( strcmpi(P.word, "CSN"      ) == 0 ) {
         isodata = CONFIG_RESET_CSN;
      } else if ( strcmpi(P.word, "RESET"    ) == 0 ) {
         isodata = CONFIG_RESET;
      } else if ( strcmpi(P.word, "RESET_DRV") == 0 ) {
         isodata = CONFIG_RESET_DRV;
      } else {
         set_errhdl( ERR_SEM_BADTOKEN );
         return( TRUE );
      }
   }

   // perform
   if ( !isolate_PNP(isodata) ) {
      printf( "\nNo cards found!\n\n" );
   } else {   
      Uport       = PNP_port;
      PNP_csn     = 0;
      PNP_csn_idx = -1;
      PNP_device  = -1;
   }

   // done
   return( TRUE );

} /* exec_isolate */


GLOBAL BOOLEAN exec_detect( VOID )
{
 INT_4  value = 0;

   // load
   if ( strcmpi(P.word, "DETECT") != 0 )
      return( FALSE );

   // get arg as int
   if ( !getINT(&value,TRUE) )
      return( TRUE );
   if ( value == 0 ) {
      value = PNP_csn;
      if ( value <= 0 ) {
         set_errhdl( ERR_SYS_DEVICE );
         return( TRUE );
      }
   }

   // done ok
   defaultport_PNP(Uport);
   if ( !loadcard_PNP((INT_2) value) ) {
      printf("\nCard #%02ld cannot be loaded\n\n", value);
   } else {
      if ( PNP_csn <= 0 )
         current_PNP( (INT_2) value );
   }
   return( TRUE );

} /* exec_detect */


GLOBAL BOOLEAN exec_status( VOID )
{
 INT_2          i;
 PNPCARD_TYP   *card;

   if ( strcmpi(P.word, "STATUS") != 0 )
      return( FALSE );
   fprintf( outfp, "\n" );
   fprintf( outfp, "PNP_port   = x%04X\n", PNP_port );
   fprintf( outfp, "PNP_csn    = %d\n",    PNP_csn  );
   fprintf( outfp, "PNP_device = %d\n",    PNP_device );
   fprintf( outfp, "#-of-cards = %d\n",    PNP_cards_len );
   fprintf( outfp, "\n" );
   for( i=0; (i < PNP_cards_len); i++ ) {
      card = &PNP_cards[i];
      fprintf( outfp, "card-#%02d: ", (i+1) );
      fprintf( outfp, "%s ", devid_PNP(card->serial) );
      fprintf( outfp, "csn=%d ", card->csn );
      if ( card->csn > 0 )
         fprintf( outfp, "devs=%d", card->dev_len );
      fprintf( outfp, "\n" );
   }
   fprintf( outfp, "\n" );
   return( TRUE );

} /* exec_status */


GLOBAL BOOLEAN exec_dump( VOID )
{
 INT_2     i;
 BOOLEAN   flag;

   // display
   if ( strcmpi(P.word, "DUMP") != 0 )
      return( FALSE );

   // dump only works if PNP_csn is set
   if ( !testcsn_PNP() )
      return( TRUE );

   // get arg
   flag = FALSE;
   if ( P.get() )
      flag = TRUE;

   // do full dump check
   if ( (flag) || (WC::streq(P.word, "CARD")) ) {
      if ( !dumprcs_PNP() ) {
         for( i=0; (i < PNP_cards[PNP_csn_idx].dev_len); i++ )
            dumpdev_PNP( i );
      }   
      return( TRUE );
   }

   // if resource
   if ( WC::streq(P.word, "RESOURCES") ) {
      dumprcs_PNP();
      return( TRUE );
   }

   // if devices
   if ( WC::streq(P.word, "DEVICE") ) {
      if ( !P.get() ) {
         if ( WC::streq(P.word, "ALL") ) {
            for( i=0; (i < PNP_cards[PNP_csn_idx].dev_len); i++ )
               dumpdev_PNP( i );
            return( TRUE );
         }
      } else {
         if ( !testdev_PNP() )
            return( TRUE );
         dumpdev_PNP( PNP_device );
         return( TRUE );
      }
   }

   // done ok
   set_errhdl( ERR_P_SETTING );
   return( TRUE );

} /* exec_dump */


STATIC VOID exec_set( VOID )
{

   // dump only works if the internal vars are set (like PNP_csn)
   if ( (!testcsn_PNP()) || (!testdev_PNP()) )
      return;
   if ( !valid_DEVICE() ) {
      set_errhdl( ERR_P_SETTING );
      return;
   }

   // get next argument
   if ( P.get() )
      goto x_done;

   // test keywords
   if ( strcmpi(P.word, "MEM") == 0 ) {
      mem_USR();
      return;
   }
   if ( strcmpi(P.word, "MEM32") == 0 ) {
      mem32_USR();
      return;
   }
   if ( strcmpi(P.word, "IOPORT") == 0 ) {
      ioport_USR();
      return;
   }
   if ( strcmpi(P.word, "IRQ") == 0 ) {
      irq_USR();
      return;
   }
   if ( strcmpi(P.word, "DMA") == 0 ) {
      dma_USR();
      return;
   }
   if ( strcmpi(P.word, "ACTIVE") == 0 ) {
      device_USR( TRUE );
      return;
   }
   if ( strcmpi(P.word, "INACTIVE") == 0 ) {
      device_USR( FALSE );
      return;
   }

x_done:

   set_errhdl( ERR_P_SETTING );
   return;

} /* exec_set */


GLOBAL BOOLEAN exec_device( VOID )
{

   // device
   if (
        (strcmpi(P.word, "DEVICE") != 0) &&
        (strcmpi(P.word, "DEV")    != 0)
      )
      return( FALSE );

   // test next keyword
   if ( P.get() )
      goto x_done;

   if ( WC::streq(P.word, "BEGIN") != 0 ) {
      beg_DEVICE();
      return( TRUE );
   }
   if ( WC::streq(P.word, "END") != 0 ) {
      end_DEVICE();
      return( TRUE );
   }
   if ( WC::streq(P.word, "SET") != 0 ) {
      exec_set();
      return( TRUE );
   }

x_done:

   // error
   set_errhdl( ERR_P_SETTING );

   // done
   return( TRUE );

} /* exec_device */


/*-------------------------------------------------------------------------*/


