/*  user2.cc : June 5, 1996  */

/* Copyright (C) 1996-1999  Sekhar Bala,
 *                          Ramchander Balasubramanian, and
 *                          Alphax Systems, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>

#include "std.h"
#include "wildcard.h"
#include "parse.h"
#include "symtab.h"
#include "input.h"
#include "main.h"
#include "pnp.h"

/*-------------------------------------------------------------------------*/

STATIC BOOLEAN           inited  = FALSE;
STATIC PNPCARD_TYP      *card    = NULL;
STATIC PNPDEV_TYP       *dev     = NULL;

/*-------------------------------------------------------------------------*/


STATIC VOID writeINTU2( INT_2 addr, INT_U2 value )
{
 UCHAR   tmp[2];

   tmp[0] = (UCHAR) (value >> 8);
   tmp[1] = (UCHAR) (value & 0x00FF);
   ADDRESS(addr+0);
   WRITE_DATA( tmp[0] );
   ADDRESS(addr+1);
   WRITE_DATA( tmp[1] );
   return;

} /* writeINTU2 */


GLOBAL BOOLEAN valid_DEVICE( VOID )
{

   return( inited );

} /* valid_DEVICE */


GLOBAL VOID beg_DEVICE( VOID )
{

   if ( inited )
      return;
   defaultport_PNP( Uport );
   initkey_PNP();
   R_SETRDPORT( PNP_port );
   card   = &PNP_cards[PNP_csn_idx];
   R_WAKE( PNP_csn );
   dev    = &(*card->dev)[PNP_device];
   R_LOGICALDEVICENUMBER;
   inited = TRUE;
   WRITE_DATA( PNP_device );
   return;

} /* beg_DEVICE */


GLOBAL VOID end_DEVICE( VOID )
{

   if ( !inited )
      return;
   R_CONFIGCONTROL;
   WRITE_DATA( CONFIG_WAIT4KEY );
   card   = NULL;
   dev    = NULL;
   inited = FALSE;
   return;

} /* end_DEVICE */


GLOBAL VOID mem_USR( VOID )
{
 INT_2      i;
 INT_2      idx;
 INT_2      tmp;
 INT_4      value;
 MEM_TYP    mem[DL_MEM_LEN];

   // get next arg
   if ( P.get() )
      goto x_err;
   idx = -1;
   for( i=0; (i < DL_MEM_LEN); i++ ) {
      mem[i].base  = 0x0;
      mem[i].flag  = 0x0;
      mem[i].limit = 0x0;
      if ( (dev->mem[i].flag & 0x01) != 0 )
         mem[i].flag = 0x01;
   }
   if ( WC::streq(P.word, "CLEAR") )
      goto x_set;

   // which MEM descriptor to set
   if ( !toINT(&value, P.word) )
      return;
   idx = (INT_2) value;
   if ( (idx < 0) || (idx >= DL_MEM_LEN) )
      goto x_err;

   // get #1
   if ( P.get() )
      goto x_err;
   if ( !toINT(&value, P.word) )
      return;
   mem[idx].base = (INT_U2) value;

   //get #2
   if ( P.get() )
      return;
   if ( !toINT(&value, P.word) )
      return;
   mem[idx].limit = (INT_U2) value;

   // get b
   if ( !P.get() ) {
      mem[idx].flag &= (~0x02);
      if        ( WC::streq(P.word, "8BIT" ) ) {
         ;
      } else if ( WC::streq(P.word, "16BIT") ) {
         mem[idx].flag |= 0x02;
      } else
         goto x_err;
   }

   // @@ flag bit 0 problem to be determined @@

x_set:

   for( i=DL_MEM_BASE; (i <= DL_MEM_LIMIT); i += DL_MEM_INC ) {
      tmp = (i - DL_MEM_BASE) / DL_MEM_INC;
      if ( (idx != -1) && (idx != tmp) )
         continue;
      if ( (dev->mem[tmp].base == 0) && (mem[tmp].base == 0) )
         continue;
      dev->mem[tmp].base  = mem[tmp].base;
      dev->mem[tmp].flag  = mem[tmp].flag;
      dev->mem[tmp].limit = mem[tmp].limit;
      writeINTU2( i, mem[tmp].base );
      ADDRESS( i+2 );
      WRITE_DATA( mem[tmp].flag );
      writeINTU2( i+3, mem[tmp].limit );
   }

   // done
   return;

x_err:

   set_errhdl( ERR_P_SETTING );
   return;

} /* mem_USR */


GLOBAL VOID mem32_USR( VOID )
{
 INT_2      i;
 INT_2      idx;
 INT_2      tmp;
 INT_U2     ubyte;
 INT_4      value;
 MEM32_TYP  mem32[DL_MEM32_LEN];

   // get next arg
   if ( P.get() )
      goto x_err;
   idx = -1;
   for( i=0; (i < DL_MEM32_LEN); i++ ) {
      mem32[i].base  = 0x0;
      mem32[i].flag  = 0x0;
      mem32[i].limit = 0x0;
      if ( (dev->mem32[i].flag & 0x01) != 0 )
         mem32[i].flag = 0x01;
   }
   if ( WC::streq(P.word, "CLEAR") )
      goto x_set;

   // which MEM descriptor to set
   if ( !toINT(&value, P.word) )
      return;
   idx = (INT_2) value;
   if ( (idx < 0) || (idx >= DL_MEM32_LEN) )
      goto x_err;

   // get #1
   if ( P.get() )
      goto x_err;
   if ( !toINT(&value, P.word) )
      return;
   mem32[idx].base = (INT_U4) value;

   //get #2
   if ( P.get() )
      return;
   if ( !toINT(&value, P.word) )
      return;
   mem32[idx].limit = (INT_U4) value;

   // get b
   if ( !P.get() ) {
      mem32[idx].flag &= (~(0x03 << 1));
      if        ( WC::streq(P.word, "8BIT" ) ) {
         ;
      } else if ( WC::streq(P.word, "16BIT") ) {
         mem32[idx].flag |= (0x01 << 1);
      } else if ( WC::streq(P.word, "32BIT") ) {
         mem32[idx].flag |= (0x03 << 1);
      } else
         goto x_err;
   }

   // @@ flag bit 0 problem to be determined @@

x_set:

   for( i=DL_MEM32_BASE; (i <= DL_MEM32_LIMIT); i += DL_MEM32_INC ) {
      tmp = (i - DL_MEM32_BASE) / DL_MEM32_INC;
      if ( i == DL_MEM32_BASE )
         i += 6;
      if ( (idx != -1) && (idx != tmp) )
         continue;
      if ( (dev->mem32[tmp].base == 0) && (mem32[tmp].base == 0) )
         continue;
      dev->mem32[tmp].base  = mem32[tmp].base;
      dev->mem32[tmp].flag  = mem32[tmp].flag;
      dev->mem32[tmp].limit = mem32[tmp].limit;
      ubyte = (INT_U2) ( mem32[tmp].base >> 16 );
      writeINTU2( i+0, ubyte );
      ubyte = (INT_U2) ( mem32[tmp].base & 0xFFFF );
      writeINTU2( i+2, ubyte );
      ADDRESS( i+4 );
      WRITE_DATA( mem32[tmp].flag );
      ubyte = (INT_U2) ( mem32[tmp].limit >> 16 );
      writeINTU2( i+5, ubyte );
      ubyte = (INT_U2) ( mem32[tmp].limit & 0xFFFF );
      writeINTU2( i+7, ubyte );
      if ( i == DL_MEM32_BASE+6 )
         i -= 6;
   }

   // done
   return;

x_err:

   set_errhdl( ERR_P_SETTING );
   return;

} /* mem32_USR */


GLOBAL VOID ioport_USR( VOID )
{
 INT_2   i;
 INT_2   tmp;
 INT_4   value;
 INT_U2  ioport[DL_IO_LEN];

   // get next tag
   if ( P.get() )
      goto x_err;
   for( i=0; (i < DL_IO_LEN); i++ )
      ioport[i] = 0x00;
   if ( WC::streq(P.word, "CLEAR") )
      goto x_set;

   // otherwise user requested cfg
   i = 0;
   while (TRUE) {
      if ( !toINT(&value,P.word) )
         return;
      if ( i >= DL_IO_LEN )
         goto x_err;
      ioport[i++] = (INT_U2) value;
      if ( (P.get()) || (P.word[0] == ';') || (P.word[0] == '#') )
         break;
      if ( P.word[0] != ',' )
         goto x_err;
      if ( P.get() )
         goto x_err;
      if ( (P.word[0] == ';') || (P.word[0] == '#') )
         break;
   }

x_set:

   for( i=DL_IO_BASE; (i <= DL_IO_LIMIT); i += DL_IO_INC ) {
      tmp = (i - DL_IO_BASE) / DL_IO_INC;
      if ( (dev->ioport[tmp] == 0) && (ioport[tmp] == 0) )
         continue;
      dev->ioport[tmp] = ioport[tmp];
      writeINTU2( i, ioport[tmp] );
   }

   // done
   return;

x_err:

   set_errhdl( ERR_P_SETTING );
   return;

} /* ioport_USR */


GLOBAL VOID irq_USR( VOID )
{
 INT_2   i;
 INT_2   idx;
 INT_2   tmp;
 INT_4   value;
 UCHAR   irq[DL_IRQ_LEN][2];

   // get next tag
   if ( P.get() )
      goto x_err;
   idx = -1;
   for( i=0; (i < DL_IRQ_LEN); i++ ) {
      irq[i][0] = 0;
      irq[i][0] = 0;
   }
   if ( WC::streq(P.word, "CLEAR") )
      goto x_set;

   // which IRQ should be set (0 or 1)
   if ( !toINT(&value, P.word) )
      return;
   idx = (INT_2) value;
   if ( (idx < 0) || (idx >= DL_IRQ_LEN) )
      goto x_err;
   if ( P.get() )
      goto x_err;

   // otherwise user requested cfg
   if ( !toINT(&value,P.word) )
      return;      
   irq[idx][0] = (UCHAR) value;
   while (TRUE) {
      if ( (P.get()) || (P.word[0] == ';') || (P.word[0] == '#') )
         break;
      if ( WC::streq(P.word, "HIGH") ) {
         irq[idx][1] |= 0x02;
         continue;
      }
      if ( WC::streq(P.word, "LOW") ) {
         irq[idx][1] &= (~0x02);
         continue;
      }
      if ( WC::streq(P.word, "LEVEL") ) {
         irq[idx][1] |= 0x01;
         continue;
      }
      if ( WC::streq(P.word, "EDGE") ) {
         irq[idx][1] &= (~0x01);
         continue;
      }
      goto x_err;
   }

x_set:

   for( i=DL_IRQ_BASE; (i <= DL_IRQ_LIMIT); i += DL_IRQ_INC ) {
      tmp = (i - DL_IRQ_BASE) / DL_IRQ_INC;
      if ( (idx != -1) && (idx != tmp) )
         continue;
      if ( (dev->irq[tmp][0] == 0) && (irq[tmp][0] == 0) )
         continue;
      dev->irq[tmp][0] = irq[tmp][0];
      dev->irq[tmp][1] = irq[tmp][1];
      ADDRESS( i+0 );
      WRITE_DATA( irq[tmp+0][0] );
      ADDRESS( i+1 );
      WRITE_DATA( irq[tmp+0][1] );
   }

   // done
   return;

x_err:

   set_errhdl( ERR_P_SETTING );
   return;

} /* irq_USR */


GLOBAL VOID dma_USR( VOID )
{
 INT_2   i;
 INT_2   idx;
 INT_2   tmp;
 INT_4   value;
 UCHAR   dma[DL_DMA_LEN];

   // get next tag
   if ( P.get() )
      goto x_err;
   for( i=0; (i < DL_DMA_LEN); i++ )
      dma[i] = 0x04;
   idx = -1;
   if ( WC::streq(P.word, "CLEAR") )
      goto x_set;

   // which dma should be set (0 or 1)
   if ( !toINT(&value,P.word) )
      return;
   idx = (INT_2) value;
   if ( (idx < 0) || (idx >= DL_DMA_LEN) )
      goto x_err;
   if ( P.get() )
      goto x_err;

   // otherwise user requested cfg
   if ( !toINT(&value,P.word) )
      return;      
   dma[idx] = (UCHAR) value;

x_set:

   for( i=DL_DMA_BASE; (i <= DL_DMA_LIMIT); i += DL_DMA_INC ) {
      tmp = (i - DL_DMA_BASE) / DL_DMA_INC;
      if ( (idx != -1) && (idx != tmp) )
         continue;
      if ( (dev->dma[tmp] == 0x04) && (dma[tmp] == 0x04) )
         continue;
      dev->dma[tmp] = dma[tmp];
      ADDRESS( i+0 );
      WRITE_DATA( dma[tmp] );
   }

   // done
   return;

x_err:

   set_errhdl( ERR_P_SETTING );
   return;

} /* dma_USR */


GLOBAL VOID device_USR( BOOLEAN on )
{
 UCHAR     udata;


   // default to off
   udata = 0x00;

   // if on...
   if ( on ) {
      // make sure range check if off
      if ( (dev->rangecheck & 0x01) != 0 ) {
         R_IORANGECHECK;
         WRITE_DATA( 0x00 );
      }
      udata = 0x01;
   }

   // set activation mode
   dev->active = udata;
   R_ACTIVATE;
   WRITE_DATA( udata );

   // done
   return;

} /* device_USR */


GLOBAL BOOLEAN test_USR( VOID )
{

   // test
   if ( strcmpi(P.word, "TEST") != 0 )
      return( FALSE );

   PNP_csn     = 1;
   PNP_csn_idx = 0;
   PNP_device  = 2;
   initkey_PNP();
   R_SETRDPORT( PNP_port );
   R_WAKE( PNP_csn );
   R_LOGICALDEVICENUMBER;
   WRITE_DATA( PNP_device );

   ADDRESS( DL_IO_BASE );
   WRITE_DATA( 0x02 );
   ADDRESS( DL_IO_BASE+1 );
   WRITE_DATA( 0x01 );

   R_ACTIVATE;
   WRITE_DATA( 0x01 );

   R_CONFIGCONTROL;
   WRITE_DATA( CONFIG_WAIT4KEY );

   // done
   return( TRUE );

} /* test_USR */


/*-------------------------------------------------------------------------*/

